@@ -28,7 +28,7 @@ import patoolib
 from renku.command.command_builder.command import inject
 from renku.command.view_model.dataset import DatasetFileViewModel, DatasetViewModel
 from renku.core import errors
-from renku.core.dataset.constant import renku_dataset_images_path, renku_pointers_path
+from renku.core.config import get_value, remove_value, set_value
 from renku.core.dataset.datasets_provenance import DatasetsProvenance
 from renku.core.dataset.pointer_file import create_external_file, is_external_file_updated, update_external_file
 from renku.core.dataset.providers.factory import ProviderFactory
@@ -36,12 +36,11 @@ from renku.core.dataset.providers.models import ProviderDataset
 from renku.core.dataset.providers.s3 import S3Credentials
 from renku.core.dataset.request_model import ImageRequestModel
 from renku.core.dataset.tag import get_dataset_by_tag, prompt_access_token, prompt_tag_selection
-from renku.core.interface.client_dispatcher import IClientDispatcher
 from renku.core.interface.dataset_gateway import IDatasetGateway
 from renku.core.interface.storage import IStorageFactory
+from renku.core.storage import check_external_storage, pull_paths_from_storage, track_paths_in_storage
 from renku.core.util import communication
 from renku.core.util.datetime8601 import local_now
-from renku.core.util.dispatcher import get_client, get_database
 from renku.core.util.git import clone_repository, get_cache_directory_for_repository, get_git_user
 from renku.core.util.metadata import is_external_file, prompt_for_credentials, read_credentials, store_credentials
 from renku.core.util.os import (
@@ -49,6 +48,7 @@ from renku.core.util.os import (
     delete_dataset_file,
     delete_path,
     get_absolute_path,
+    get_files,
     get_safe_relative_path,
     hash_file,
     is_path_empty,
@@ -60,12 +60,12 @@ from renku.core.util.urls import get_slug
 from renku.core.util.util import NO_VALUE, NoValueType
 from renku.domain_model.dataset import Dataset, DatasetDetailsJson, DatasetFile, RemoteEntity, is_dataset_name_valid
 from renku.domain_model.enums import ConfigFilter
+from renku.domain_model.project_context import project_context
 from renku.domain_model.provenance.agent import Person
 from renku.domain_model.provenance.annotation import Annotation
 from renku.infrastructure.immutable import DynamicProxy
 
 if TYPE_CHECKING:
-    from renku.core.management.client import LocalClient
     from renku.infrastructure.repository import Repository
 
 
@@ -128,11 +128,9 @@ def create_dataset(
     Returns:
         Dataset: The created dataset.
     """
-    client = get_client()
-
     if not creators:
         creators = []
-        user = get_git_user(client.repository)
+        user = get_git_user(repository=project_context.repository)
 
         if user:
             creators.append(user)
@@ -157,7 +155,7 @@ def create_dataset(
 
     if datadir:
         try:
-            datadir = get_safe_relative_path(datadir, client.path)
+            datadir = get_safe_relative_path(datadir, project_context.path)
         except ValueError as e:
             raise errors.ParameterError("Datadir must be inside repository.") from e
 
@@ -168,18 +166,20 @@ def create_dataset(
         description=description,
         creators=creators,
         keywords=keywords,
-        project_id=client.project.id,
+        project_id=project_context.project.id,
         annotations=annotations,
         storage=storage,
         datadir=datadir,
     )
 
     if images:
-        set_dataset_images(client, dataset, images)
+        set_dataset_images(dataset=dataset, images=images)
 
     if storage:
         provider = ProviderFactory.get_create_provider(uri=storage)
         provider.on_create(dataset=dataset)
+    else:
+        add_datadir_files_to_dataset(dataset)
 
     if update_provenance:
         datasets_provenance.add_or_update(dataset)
@@ -187,16 +187,15 @@ def create_dataset(
     return dataset
 
 
-@inject.autoparams("client_dispatcher")
 def edit_dataset(
     name: str,
     title: Optional[Union[str, NoValueType]],
     description: Optional[Union[str, NoValueType]],
     creators: Optional[Union[List[Person], NoValueType]],
-    client_dispatcher: IClientDispatcher,
     keywords: Optional[Union[List[str], NoValueType]] = NO_VALUE,
     images: Optional[Union[List[ImageRequestModel], NoValueType]] = NO_VALUE,
-    custom_metadata: Optional[Union[Dict, NoValueType]] = NO_VALUE,
+    custom_metadata: Optional[Union[Dict, List[Dict], NoValueType]] = NO_VALUE,
+    custom_metadata_source: Optional[Union[str, NoValueType]] = NO_VALUE,
 ):
     """Edit dataset metadata.
 
@@ -205,17 +204,17 @@ def edit_dataset(
         title(Optional[Union[str, NoValueType]]): New title for the dataset.
         description(Optional[Union[str, NoValueType]]): New description for the dataset.
         creators(Optional[Union[List[Person], NoValueType]]): New creators for the dataset.
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         keywords(Optional[Union[List[str], NoValueType]]): New keywords for dataset (Default value = ``NO_VALUE``).
         images(Optional[Union[List[ImageRequestModel], NoValueType]]): New images for dataset
             (Default value = ``NO_VALUE``).
-        custom_metadata(Optional[Union[Dict, NoValueType]]): Custom JSON-LD metadata (Default value = ``NO_VALUE``).
+        custom_metadata(Optional[Union[Dict, List[Dict], NoValueType]]): Custom JSON-LD metadata
+            (Default value = ``NO_VALUE``).
+        custom_metadata_source(Optional[Union[str, NoValueType]]): The custom metadata source
+            (Default value = ``NO_VALUE``).
 
     Returns:
         bool: True if updates were performed.
     """
-    client = client_dispatcher.current_client
-
     if isinstance(title, str):
         title = title.strip()
 
@@ -243,7 +242,7 @@ def edit_dataset(
     if images == NO_VALUE:
         images_updated = False
     else:
-        images_updated = set_dataset_images(client, dataset, cast(Optional[List[ImageRequestModel]], images))
+        images_updated = set_dataset_images(dataset=dataset, images=cast(Optional[List[ImageRequestModel]], images))
 
     if images_updated:
         updated["images"] = (
@@ -251,21 +250,23 @@ def edit_dataset(
         )
 
     if custom_metadata is not NO_VALUE:
-        update_dataset_custom_metadata(dataset, cast(Optional[Dict], custom_metadata))
+        update_dataset_custom_metadata(
+            dataset,
+            cast(Optional[Union[Dict, List[Dict]]], custom_metadata),
+            cast(Optional[str], custom_metadata_source),
+        )
         updated["custom_metadata"] = custom_metadata
 
     if not updated:
         return []
 
     datasets_provenance = DatasetsProvenance()
-    datasets_provenance.add_or_update(dataset, creator=get_git_user(client.repository))
+    datasets_provenance.add_or_update(dataset, creator=get_git_user(project_context.repository))
 
     return updated
 
 
-@inject.autoparams("client_dispatcher")
 def list_dataset_files(
-    client_dispatcher: IClientDispatcher,
     datasets: List[str] = None,
     tag: Optional[str] = None,
     creators=None,
@@ -275,7 +276,6 @@ def list_dataset_files(
     """List dataset files.
 
     Args:
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         datasets(List[str]): Datasets to list files for (Default value = None).
         tag(str): Tag to filter by (Default value = None).
         creators: Creators to filter by (Default value = None).
@@ -287,8 +287,6 @@ def list_dataset_files(
     """
     from renku.command.format.dataset_files import get_lfs_tracking_and_file_sizes
 
-    client = client_dispatcher.current_client
-
     records = filter_dataset_files(
         names=datasets, tag=tag, creators=creators, include=include, exclude=exclude, immutable=True
     )
@@ -298,7 +296,7 @@ def list_dataset_files(
         record.dataset_id = record.dataset.id
         record.creators_csv = record.dataset.creators_csv
         record.creators_full_csv = record.dataset.creators_full_csv
-        record.full_path = client.path / record.entity.path
+        record.full_path = project_context.path / record.entity.path
         record.path = record.entity.path
         record.name = Path(record.entity.path).name
         record.added = record.date_added
@@ -308,21 +306,19 @@ def list_dataset_files(
     return records
 
 
-@inject.autoparams()
-def file_unlink(name, include, exclude, client_dispatcher: IClientDispatcher, yes=False):
+def file_unlink(name, include, exclude, yes=False):
     """Remove matching files from a dataset.
 
     Args:
         name: Dataset name.
         include: Include filter for files.
         exclude: Exclude filter for files.
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         yes: Whether to skip user confirmation or not (Default value = False).
 
     Returns:
         List[DynamicProxy]: List of files that were removed.
     """
-    client = client_dispatcher.current_client
+    repository = project_context.repository
 
     if not include and not exclude:
         raise errors.ParameterError("include or exclude filters not specified.")
@@ -368,9 +364,9 @@ def file_unlink(name, include, exclude, client_dispatcher: IClientDispatcher, ye
                     f"Dataset file {path_file} could not be removed from {dataset_datadir} because of {err}."
                 )
             else:
-                client.repository.add(path_file)
+                repository.add(path_file)
 
-    datasets_provenance.add_or_update(dataset, creator=get_git_user(client.repository))
+    datasets_provenance.add_or_update(dataset, creator=get_git_user(repository))
 
     return records
 
@@ -386,17 +382,14 @@ def remove_dataset(name):
     datasets_provenance.remove(dataset=dataset)
 
 
-@inject.autoparams()
-def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatcher, **kwargs):
+def export_dataset(name, provider_name, tag, **kwargs):
     """Export data to 3rd party provider.
 
     Args:
         name: Name of dataset to export.
         provider_name: Provider to use for export.
         tag: Dataset tag from which to export.
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
     """
-    client = client_dispatcher.current_client
     datasets_provenance = DatasetsProvenance()
 
     provider_name = provider_name.lower()
@@ -427,9 +420,6 @@ def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatche
 
     dataset = cast(Dataset, DynamicProxy(dataset))
 
-    data_dir = dataset.get_datadir()
-    dataset.data_dir = data_dir
-
     exporter = provider.get_exporter(dataset=dataset, tag=selected_tag, **kwargs)
 
     if exporter.requires_access_token():
@@ -446,9 +436,9 @@ def export_dataset(name, provider_name, tag, client_dispatcher: IClientDispatche
         exporter.set_access_token(access_token)
 
     try:
-        destination = exporter.export(client=client)
+        destination = exporter.export()
     except errors.AuthenticationError:
-        client.remove_value(provider_name, config_key_secret, global_only=True)
+        remove_value(provider_name, config_key_secret, global_only=True)
         raise
 
     communication.echo(f"Exported to: {destination}")
@@ -479,23 +469,21 @@ def import_dataset(
     """
     from renku.core.dataset.dataset_add import add_to_dataset
 
-    client = get_client()
-
     def confirm_download(files):
         if yes:
             return
 
-        headers = {"checksum": "checksum", "filename": "name", "size_in_mb": "size (b)", "filetype": "type"}
+        headers = {"checksum": "checksum", "filename": "name", "filesize_str": "size", "filetype": "type"}
         communication.echo(tabulate(files, headers=headers, floatfmt=".2f"))
         communication.confirm("Do you wish to download this version?", abort=True, warning=True)
 
-    def calculate_total_size(files):
-        total_size = 0.0
+    def calculate_total_size(files) -> int:
+        total_size = 0
         for file in files:
-            if file.size_in_mb is not None:
-                total_size += file.size_in_mb
+            if file.filesize is not None:
+                total_size += file.filesize
 
-        return total_size * 2**20
+        return total_size
 
     def remove_files(dataset):
         """Remove files that exist in ``previous_dataset`` but not in ``dataset``.
@@ -511,7 +499,7 @@ def import_dataset(
         deleted_paths = previous_paths - current_paths
 
         for path in deleted_paths:
-            delete_dataset_file(client.path / path, follow_symlinks=True)
+            delete_dataset_file(project_context.path / path, follow_symlinks=True)
 
     provider = ProviderFactory.get_import_provider(uri)
 
@@ -538,7 +526,7 @@ def import_dataset(
         raise errors.ParameterError("Can't specify datadir when updating a previously imported dataset.")
     elif datadir:
         try:
-            datadir = get_safe_relative_path(datadir, client.path)
+            datadir = get_safe_relative_path(datadir, project_context.path)
         except ValueError as e:
             raise errors.ParameterError("Datadir must be inside repository.") from e
 
@@ -566,7 +554,7 @@ def import_dataset(
     importer.tag_dataset(name)
     importer.copy_extra_metadata(new_dataset)
 
-    get_database().commit()
+    project_context.database.commit()
 
 
 @inject.autoparams()
@@ -583,7 +571,6 @@ def update_datasets(
     check_data_directory: bool,
     update_all: bool,
     dry_run: bool,
-    client_dispatcher: IClientDispatcher,
     dataset_gateway: IDatasetGateway,
 ) -> Tuple[List[DatasetViewModel], List[DatasetFileViewModel]]:
     """Update dataset files.
@@ -601,7 +588,6 @@ def update_datasets(
         check_data_directory(bool): Whether to check the dataset's data directory for new files.
         update_all(bool): Whether to update all datasets.
         dry_run(bool): Whether to return a preview of what would be updated.
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         dataset_gateway(IDatasetGateway): Injected dataset gateway.
     """
     from renku.core.dataset.providers.renku import RenkuProvider
@@ -609,8 +595,6 @@ def update_datasets(
     if not update_all and not names and not include and not exclude and not dry_run:
         raise errors.ParameterError("No update criteria is specified")
 
-    client = client_dispatcher.current_client
-
     imported_dataset_updates: List[Dataset] = []
 
     all_datasets = dataset_gateway.get_all_active_datasets()
@@ -725,7 +709,7 @@ def update_datasets(
     deleted_files: List[DynamicProxy] = []
 
     if external_files and not no_external:
-        updated = update_external_files(client, external_files, dry_run=dry_run)
+        updated = update_external_files(external_files, dry_run=dry_run)
         updated_files.extend(updated)
 
     if git_files and not no_remote:
@@ -745,12 +729,13 @@ def update_datasets(
         if deleted_files and not delete:
             communication.echo("Some files are deleted: Pass '--delete' to remove them from datasets' metadata")
         if updated_files or (deleted_files and delete):
-            file_paths = {str(client.path / f.entity.path) for f in updated_files}
+            file_paths = {str(project_context.path / f.entity.path) for f in updated_files}
             # Force-add to include possible ignored files that are in datasets
-            client.repository.add(*file_paths, force=True)
-            client.repository.add(renku_pointers_path(client), force=True)
+            repository = project_context.repository
+            repository.add(*file_paths, force=True)
+            repository.add(project_context.pointers_path, force=True)
 
-            _update_datasets_files_metadata(client, updated_files, deleted_files, delete)
+            _update_datasets_files_metadata(updated_files, deleted_files, delete)
 
         message = f"Updated {len(updated_files)} files"
         if delete:
@@ -794,11 +779,36 @@ def show_dataset(name: str, tag: Optional[str] = None):
     return DatasetDetailsJson().dump(dataset)
 
 
-def set_dataset_images(client: "LocalClient", dataset: Dataset, images: Optional[List[ImageRequestModel]]):
+def add_datadir_files_to_dataset(dataset: Dataset) -> None:
+    """Add all files in a datasets data directory to the dataset.
+
+    Args:
+        dataset(Dataset): The dataset to add data dir files to.
+    """
+    datadir = get_safe_relative_path(dataset.get_datadir(), project_context.path)
+
+    if datadir.exists():
+        # NOTE: Add existing files to dataset
+        dataset_files: List[DatasetFile] = []
+        files: List[Path] = []
+        for file in get_files(datadir):
+            files.append(file)
+            dataset_files.append(DatasetFile.from_path(path=file, source=file))
+
+        if not dataset_files:
+            return
+
+        if check_external_storage():
+            track_paths_in_storage(*files)
+        project_context.repository.add(*files)
+
+        dataset.add_or_update_files(dataset_files)
+
+
+def set_dataset_images(dataset: Dataset, images: Optional[List[ImageRequestModel]]):
     """Set a dataset's images.
 
     Args:
-        client("LocalClient"): The ``LocalClient``.
         dataset(Dataset): The dataset to set images on.
         images(List[ImageRequestModel]): The images to set.
 
@@ -808,7 +818,7 @@ def set_dataset_images(client: "LocalClient", dataset: Dataset, images: Optional
     if not images:
         images = []
 
-    image_folder = renku_dataset_images_path(client) / dataset.initial_identifier
+    image_folder = project_context.dataset_images_path / dataset.initial_identifier
     image_folder.mkdir(exist_ok=True, parents=True)
     previous_images = dataset.images or []
 
@@ -835,32 +845,41 @@ def set_dataset_images(client: "LocalClient", dataset: Dataset, images: Optional
 
         path = prev.content_url
         if not os.path.isabs(path):
-            path = os.path.normpath(os.path.join(client.path, path))
+            path = os.path.normpath(os.path.join(project_context.path, path))
 
         os.remove(path)
 
     return images_updated or dataset.images != previous_images
 
 
-def update_dataset_custom_metadata(dataset: Dataset, custom_metadata: Optional[Dict]):
+def update_dataset_custom_metadata(
+    dataset: Dataset,
+    custom_metadata: Optional[Union[Dict, List[Dict]]],
+    custom_metadata_source: Optional[str],
+):
     """Update custom metadata on a dataset.
 
     Args:
         dataset(Dataset): The dataset to update.
         custom_metadata(Dict): Custom JSON-LD metadata to set.
+        custom_metadata_source(str): The source field for the custom metadata.
     """
 
-    existing_metadata = [a for a in dataset.annotations if a.source != "renku"]
+    existing_metadata = [a for a in dataset.annotations if a.source != custom_metadata_source]
 
-    if custom_metadata is not None:
-        existing_metadata.append(Annotation(id=Annotation.generate_id(), body=custom_metadata, source="renku"))
+    if custom_metadata is not None and custom_metadata_source is not None:
+        if isinstance(custom_metadata, dict):
+            custom_metadata = [custom_metadata]
+        for icustom_metadata in custom_metadata:
+            existing_metadata.append(
+                Annotation(id=Annotation.generate_id(), body=icustom_metadata, source=custom_metadata_source)
+            )
 
     dataset.annotations = existing_metadata
 
 
-@inject.autoparams("client_dispatcher", "dataset_gateway")
+@inject.autoparams("dataset_gateway")
 def move_files(
-    client_dispatcher: IClientDispatcher,
     dataset_gateway: IDatasetGateway,
     files: Dict[Path, Path],
     to_dataset_name: Optional[str] = None,
@@ -868,13 +887,10 @@ def move_files(
     """Move files and their metadata from one or more datasets to a target dataset.
 
     Args:
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         dataset_gateway(IDatasetGateway):Injected dataset gateway.
         files(Dict[Path, Path]): Files to move
         to_dataset_name(Optional[str], optional): Target dataset (Default value = None)
     """
-    client = client_dispatcher.current_client
-
     datasets = [d.copy() for d in dataset_gateway.get_all_active_datasets()]
 
     to_dataset: Optional[Dataset] = None
@@ -887,10 +903,10 @@ def move_files(
     communication.start_progress(progress_name, total=len(files))
     try:
         for src, dst in files.items():
-            src = src.relative_to(client.path)
-            dst = dst.relative_to(client.path)
+            src = src.relative_to(project_context.path)
+            dst = dst.relative_to(project_context.path)
             # NOTE: Files are moved at this point, so, we can use dst
-            new_dataset_file = DatasetFile.from_path(client, dst)
+            new_dataset_file = DatasetFile.from_path(dst)
 
             for dataset in datasets:
                 removed = dataset.unlink_file(src, missing_ok=True)
@@ -901,13 +917,13 @@ def move_files(
 
                     if not to_dataset and (
                         new_dataset_file.is_external
-                        or is_subpath(client.path / dst, client.path / dataset.get_datadir(client))
+                        or is_subpath(project_context.path / dst, project_context.path / dataset.get_datadir())
                     ):
                         dataset.add_or_update_files(new_dataset_file)
 
                 # NOTE: Update dataset if it contains a destination that is being overwritten
                 modified = dataset.find_file(dst)
-                added = is_subpath(client.path / dst, client.path / dataset.get_datadir(client))
+                added = is_subpath(project_context.path / dst, project_context.path / dataset.get_datadir())
                 if modified or added:
                     modified_datasets[dataset.name] = dataset
                     dataset.add_or_update_files(new_dataset_file)
@@ -921,27 +937,24 @@ def move_files(
 
     datasets_provenance = DatasetsProvenance()
     modified_dataset_values = list(modified_datasets.values())
+    creator = get_git_user(repository=project_context.repository)
     for modified_dataset in modified_dataset_values:
-        datasets_provenance.add_or_update(modified_dataset, creator=get_git_user(client.repository))
+        datasets_provenance.add_or_update(modified_dataset, creator=creator)
     if to_dataset and to_dataset not in modified_dataset_values:
-        datasets_provenance.add_or_update(to_dataset, creator=get_git_user(client.repository))
+        datasets_provenance.add_or_update(to_dataset, creator=creator)
 
 
-@inject.autoparams("client_dispatcher")
 def update_dataset_local_files(
-    client_dispatcher: IClientDispatcher, records: List[DynamicProxy], check_data_directory: bool
+    records: List[DynamicProxy], check_data_directory: bool
 ) -> Tuple[List[DynamicProxy], List[DynamicProxy], List[DynamicProxy]]:
     """Update files metadata from the git history.
 
     Args:
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         records(List[DynamicProxy]): File records to update.
         check_data_directory(bool): Whether to check the dataset's data directory for new files.
     Returns:
         Tuple[List[DynamicProxy], List[DynamicProxy]]: Tuple of updated and deleted file records.
     """
-    client = client_dispatcher.current_client
-
     updated_files: List[DynamicProxy] = []
     deleted_files: List[DynamicProxy] = []
     new_files: List[DynamicProxy] = []
@@ -958,14 +971,14 @@ def update_dataset_local_files(
             if file.based_on or file.is_external:
                 continue
 
-            if not (client.path / file.entity.path).exists():
+            if not (project_context.path / file.entity.path).exists():
                 deleted_files.append(file)
                 continue
 
             check_paths.append(file.entity.path)
             records_to_check.append(file)
 
-        checksums = client.repository.get_object_hashes(check_paths)
+        checksums = project_context.repository.get_object_hashes(check_paths)
 
         for file in records_to_check:
             current_checksum = checksums.get(file.entity.path)
@@ -987,18 +1000,11 @@ def update_dataset_local_files(
     return updated_files, deleted_files, new_files
 
 
-def _update_datasets_files_metadata(
-    client: "LocalClient",
-    updated_files: List[DynamicProxy],
-    deleted_files: List[DynamicProxy],
-    delete: bool,
-):
+def _update_datasets_files_metadata(updated_files: List[DynamicProxy], deleted_files: List[DynamicProxy], delete: bool):
     modified_datasets = {}
 
     for file in updated_files:
-        new_file = DatasetFile.from_path(
-            client=client, path=file.entity.path, based_on=file.based_on, source=file.source
-        )
+        new_file = DatasetFile.from_path(path=file.entity.path, based_on=file.based_on, source=file.source)
         modified_datasets[file.dataset.name] = file.dataset
         file.dataset.add_or_update_files(new_file)
 
@@ -1009,30 +1015,24 @@ def _update_datasets_files_metadata(
 
     datasets_provenance = DatasetsProvenance()
     for dataset in modified_datasets.values():
-        datasets_provenance.add_or_update(dataset, creator=get_git_user(client.repository))
+        datasets_provenance.add_or_update(dataset, creator=get_git_user(repository=project_context.repository))
 
 
-@inject.autoparams("client_dispatcher")
 def update_dataset_git_files(
-    client_dispatcher: IClientDispatcher, files: List[DynamicProxy], ref: str, delete: bool, dry_run: bool
+    files: List[DynamicProxy], ref: Optional[str], delete: bool, dry_run: bool
 ) -> Tuple[List[DynamicProxy], List[DynamicProxy]]:
     """Update files and dataset metadata according to their remotes.
 
     Args:
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         files(List[DynamicProxy]): List of files to be updated.
-        ref(str): Reference to use for update.
+        ref(Optional[str]): Reference to use for update.
         delete(bool, optional): Indicates whether to delete files or not (Default value = False).
         dry_run(bool): Whether to perform update or only print changes.
 
     Returns:
         Tuple[List[DynamicProxy], List[DynamicProxy]]: Tuple of updated and deleted file records.
     """
-    from renku.core.management.client import LocalClient
-
-    client = client_dispatcher.current_client
-
-    visited_repos: Dict[str, Tuple["Repository", LocalClient]] = {}
+    visited_repos: Dict[str, "Repository"] = {}
     updated_files: List[DynamicProxy] = []
     deleted_files: List[DynamicProxy] = []
 
@@ -1048,34 +1048,32 @@ def update_dataset_git_files(
             based_on = file.based_on
             url = based_on.url
             if url in visited_repos:
-                remote_repository, remote_client = visited_repos[url]
+                remote_repository = visited_repos[url]
             else:
                 communication.echo(msg="Cloning remote repository...")
-                remote_repository = clone_repository(
-                    url=url, path=get_cache_directory_for_repository(client=client, url=url), checkout_revision=ref
-                )
-                remote_client = LocalClient(path=remote_repository.path)
-                visited_repos[url] = remote_repository, remote_client
+                path = get_cache_directory_for_repository(url=url)
+                remote_repository = clone_repository(url=url, path=path, checkout_revision=ref)
+                visited_repos[url] = remote_repository
 
             checksum = remote_repository.get_object_hash(path=based_on.path, revision="HEAD")
             found = checksum is not None
             changed = found and based_on.checksum != checksum
 
             src = remote_repository.path / based_on.path
-            dst = client.renku_path.parent / file.entity.path
+            dst = project_context.metadata_path.parent / file.entity.path
 
             if not found:
                 if not dry_run and delete:
                     delete_dataset_file(dst, follow_symlinks=True)
-                    client.repository.add(dst, force=True)
+                    project_context.repository.add(dst, force=True)
                 deleted_files.append(file)
             elif changed:
                 if not dry_run:
                     # Fetch file if it is tracked by Git LFS
-                    remote_client.pull_paths_from_storage(remote_client.path / based_on.path)
-                    if is_external_file(path=src, client_path=remote_client.path):
+                    pull_paths_from_storage(remote_repository, remote_repository.path / based_on.path)
+                    if is_external_file(path=src, project_path=remote_repository.path):
                         delete_dataset_file(dst, follow_symlinks=True)
-                        create_external_file(client=client, target=src.resolve(), path=dst)
+                        create_external_file(target=src.resolve(), path=dst)
                     else:
                         shutil.copy(src, dst)
                     file.based_on = RemoteEntity(
@@ -1092,11 +1090,10 @@ def update_dataset_git_files(
     return updated_files, deleted_files
 
 
-def update_external_files(client: "LocalClient", records: List[DynamicProxy], dry_run: bool) -> List[DynamicProxy]:
+def update_external_files(records: List[DynamicProxy], dry_run: bool) -> List[DynamicProxy]:
     """Update files linked to external storage.
 
     Args:
-        client("LocalClient"): The ``LocalClient``.
         records(List[DynamicProxy]): File records to update.
         dry_run(bool): Whether to return a preview of what would be updated.
     """
@@ -1105,7 +1102,7 @@ def update_external_files(client: "LocalClient", records: List[DynamicProxy], dr
     for file in records:
         if file.is_external:
             try:
-                updated, checksum = is_external_file_updated(client_path=client.path, path=file.entity.path)
+                updated, checksum = is_external_file_updated(project_path=project_context.path, path=file.entity.path)
             except errors.ExternalFileNotFound as e:
                 if not dry_run:
                     raise
@@ -1114,15 +1111,14 @@ def update_external_files(client: "LocalClient", records: List[DynamicProxy], dr
 
             if updated:
                 if not dry_run:
-                    update_external_file(client=client, path=file.entity.path, checksum=checksum)
+                    update_external_file(path=file.entity.path, checksum=checksum)
                 updated_files.append(file)
 
     return updated_files
 
 
-@inject.autoparams("client_dispatcher", "dataset_gateway")
+@inject.autoparams("dataset_gateway")
 def filter_dataset_files(
-    client_dispatcher: IClientDispatcher,
     dataset_gateway: IDatasetGateway,
     names: Optional[List[str]] = None,
     tag: Optional[str] = None,
@@ -1136,7 +1132,6 @@ def filter_dataset_files(
     """Filter dataset files by specified filters.
 
     Args:
-        client_dispatcher(IClientDispatcher): Injected client dispatcher.
         dataset_gateway(IDatasetGateway):Injected dataset gateway.
         names(Optional[List[str]]): Filter by specified dataset names (Default value = None).
         tag(Optional[str]): Filter by specified tag (Default value = None).
@@ -1166,8 +1161,6 @@ def filter_dataset_files(
 
         return True
 
-    client = client_dispatcher.current_client
-
     if isinstance(creators, str):
         creators_set = set(creators.split(","))
     elif isinstance(creators, list) or isinstance(creators, tuple):
@@ -1207,21 +1200,19 @@ def filter_dataset_files(
 
             record = DynamicProxy(file)
             record.dataset = dataset
-            record.client = client
             records.append(record)
 
         if not check_data_directory:
             continue
 
-        for root, _, files in os.walk(client.path / dataset.get_datadir()):
+        for root, _, files in os.walk(project_context.path / dataset.get_datadir()):
             current_folder = Path(root)
             for current_file in files:
-                file_path = get_safe_relative_path(current_folder / current_file, client.path)
+                file_path = get_safe_relative_path(current_folder / current_file, project_context.path)
                 if should_include(file_path) and not dataset.find_file(file_path):
                     # New file in dataset folder
-                    record = DynamicProxy(DatasetFile.from_path(client, file_path))
+                    record = DynamicProxy(DatasetFile.from_path(file_path))
                     record.dataset = dataset
-                    record.client = client
                     records.append(record)
 
     if unused_names:
@@ -1231,17 +1222,15 @@ def filter_dataset_files(
     return sorted(records, key=lambda r: r.date_added)
 
 
-@inject.autoparams("client_dispatcher", "storage_factory")
-def pull_external_data(
-    name: str, client_dispatcher: IClientDispatcher, storage_factory: IStorageFactory, location: Optional[Path] = None
-) -> None:
+@inject.autoparams("storage_factory")
+def pull_external_data(name: str, storage_factory: IStorageFactory, location: Optional[Path] = None) -> None:
     """Pull/copy data for an external storage to a dataset's data directory or a specified location.
 
     Args:
         name(str): Name of the dataset
+        storage_factory(IStorageFactory):Injected storage factory.
         location(Optional[Path]): A directory to copy data to (Default value = None).
     """
-    client = client_dispatcher.current_client
     datasets_provenance = DatasetsProvenance()
 
     dataset = datasets_provenance.get_by_name(name=name, strict=True)
@@ -1251,7 +1240,7 @@ def pull_external_data(
         return
 
     # NOTE: Try to unmount the path in case it was mounted before
-    unmount_path(client.path / dataset.get_datadir())
+    unmount_path(project_context.path / dataset.get_datadir())
 
     create_symlinks = True
     destination: Union[Path, str]
@@ -1263,7 +1252,7 @@ def pull_external_data(
         if stored_location:
             destination = stored_location
         else:
-            destination = client.path
+            destination = project_context.path
             create_symlinks = False
 
     provider = ProviderFactory.get_pull_provider(uri=dataset.storage)
@@ -1297,7 +1286,7 @@ def pull_external_data(
             updated_files.append(new_file)
 
             if create_symlinks:
-                symlink_path = client.path / file.entity.path
+                symlink_path = project_context.path / file.entity.path
                 symlink_path.parent.mkdir(parents=True, exist_ok=True)
                 create_symlink(path=path, symlink_path=symlink_path, overwrite=True)
 
@@ -1305,28 +1294,26 @@ def pull_external_data(
     store_dataset_data_location(dataset=dataset, location=location)
 
     if updated_files:
-        _update_datasets_files_metadata(client, updated_files=updated_files, deleted_files=[], delete=False)
+        _update_datasets_files_metadata(updated_files=updated_files, deleted_files=[], delete=False)
 
 
 def store_dataset_data_location(dataset: Dataset, location: Optional[Path]) -> None:
     """Store data location for a dataset in the config file."""
-    client = get_client()
-
     section = "dataset-locations"
     key = dataset.name
 
     if not location:
-        client.remove_value(section=section, key=key)
+        remove_value(section=section, key=key)
     else:
-        client.set_value(section=section, key=key, value=get_absolute_path(location))
+        set_value(section=section, key=key, value=get_absolute_path(location))
 
 
 def read_dataset_data_location(dataset: Dataset) -> Optional[str]:
     """Read data location for a dataset in the config file."""
-    return get_client().get_value(section="dataset-locations", key=dataset.name, config_filter=ConfigFilter.LOCAL_ONLY)
+    return get_value(section="dataset-locations", key=dataset.name, config_filter=ConfigFilter.LOCAL_ONLY)
 
 
-@inject.autoparams("client_dispatcher", "storage_factory")
+@inject.autoparams("storage_factory")
 def mount_external_storage(
     name: str,
     existing: Optional[Path],
@@ -1379,10 +1366,7 @@ def unmount_external_storage(name: str) -> None:
     unmount_path(datadir)
 
 
-@inject.autoparams("client_dispatcher")
-def _get_dataset_with_external_storage(name: str, client_dispatcher: IClientDispatcher) -> Tuple[Dataset, Path]:
-
-    client = client_dispatcher.current_client
+def _get_dataset_with_external_storage(name: str) -> Tuple[Dataset, Path]:
     datasets_provenance = DatasetsProvenance()
 
     dataset = datasets_provenance.get_by_name(name=name, strict=True)
@@ -1390,6 +1374,6 @@ def _get_dataset_with_external_storage(name: str, client_dispatcher: IClientDisp
     if not dataset.storage:
         raise errors.ParameterError(f"Dataset '{name}' doesn't have a storage backend")
 
-    datadir = client.path / dataset.get_datadir()
+    datadir = project_context.path / dataset.get_datadir()
 
     return dataset, datadir
