@@ -41,6 +41,7 @@ from renku.core.util.os import get_absolute_path, get_relative_path, is_subpath
 from renku.core.workflow.types import PATH_OBJECTS, Directory, File
 from renku.domain_model.datastructures import DirectoryTree
 from renku.domain_model.project_context import project_context
+from renku.domain_model.provenance.agent import Person
 from renku.domain_model.workflow.parameter import (
     DIRECTORY_MIME_TYPE,
     CommandInput,
@@ -699,6 +700,7 @@ class PlanFactory:
         name: Optional[str] = None,
         description: Optional[str] = None,
         keywords: Optional[List[str]] = None,
+        creators: Optional[List[Person]] = None,
     ) -> Plan:
         """Return an instance of ``Plan`` based on this factory."""
         plan = Plan(
@@ -712,6 +714,7 @@ class PlanFactory:
             parameters=self.parameters,
             project_id=project_gateway.get_project().id,
             success_codes=self.success_codes,
+            creators=creators,
         )
 
         pm = get_plugin_manager()
