@@ -22,7 +22,7 @@ from urllib3.exceptions import HTTPError
 
 from renku.command.dataset import add_to_dataset_command, import_dataset_command
 from renku.core import errors
-from renku.core.util.contexts import click_context
+from renku.core.util.contexts import renku_project_context
 from renku.core.util.git import push_changes
 from renku.domain_model.git import GitURL
 from renku.infrastructure.repository import Repository
@@ -55,7 +55,7 @@ def dataset_import(
     try:
         worker_log.debug(f"retrieving metadata for project {project_id}")
         project = cache.get_project(user, project_id)
-        with click_context(project.abs_path, "dataset_import"):
+        with renku_project_context(project.abs_path):
             worker_log.debug(f"project found in cache - importing dataset {dataset_uri}")
             communicator = ServiceCallback(user_job=user_job)
 
@@ -107,7 +107,7 @@ def dataset_add_remote_file(cache, user, user_job_id, project_id, create_dataset
         worker_log.debug(f"checking metadata for project {project_id}")
         project = cache.get_project(user, project_id)
 
-        with click_context(project.abs_path, "dataset_add_remote_file"):
+        with renku_project_context(project.abs_path):
             urls = url if isinstance(url, list) else [url]
 
             worker_log.debug(f"adding files {urls} to dataset {name}")
